@@ -23,10 +23,10 @@ __all__ = [
 		]
 
 __copyright__ = """
-2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
+2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 """
 
-__version__ = "0.5.2"
+__version__ = "0.8.1"
 
 repo_root = pathlib.Path(__file__).parent
 install_requires = (repo_root / "requirements.txt").read_text(encoding="utf-8").split('\n')
