@@ -5,7 +5,7 @@
 Utilities for working with :pep:`508` requirements.
 """
 #
-#  Copyright © 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
+#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -153,8 +153,8 @@ class ComparableRequirement(Requirement):
 		return hash((
 				self.name or '',
 				self.url or '',
-				self.specifier or '',
-				self.marker or '',
+				str(self.specifier) or '',
+				str(self.marker) or '',
 				*(self.extras or ()),
 				))
 
@@ -270,13 +270,12 @@ def combine_requirements(
 	"""
 	Combine duplicated requirements in a list.
 
+	.. versionchanged:: 0.2.1  Added the ``normalize_func`` keyword-only argument.
+	.. versionchanged:: 0.3.1  Requirements are no longer combined if their markers differ.
+
 	:param requirement: A single requirement, or an iterable of requirements.
 	:param requirements: Additional requirements.
 	:param normalize_func: Function to use to normalize the names of requirements.
-
-	.. versionchanged:: 0.2.1  Added the ``normalize_func`` keyword-only argument.
-
-	.. versionchanged:: 0.3.1  Requirements are no longer combined if their markers differ.
 	"""
 
 	if isinstance(requirement, Iterable):
@@ -295,7 +294,7 @@ def combine_requirements(
 		if req.name in merged_requirements:
 			other_req = merged_requirements[merged_requirements.index(req.name)]  # type: ignore
 
-			if req.marker != other_req.marker:
+			if str(req.marker) != str(other_req.marker):
 				merged_requirements.append(req)
 			else:
 				other_req.specifier &= req.specifier
@@ -341,15 +340,14 @@ def read_requirements(
 	"""
 	Reads :pep:`508` requirements from the given file.
 
+	.. versionchanged:: 0.2.0 Added the ``include_invalid`` option.
+	.. versionchanged:: 0.2.1 Added the ``normalize_func`` keyword-only argument.
+
 	:param req_file:
 	:param include_invalid: If :py:obj:`True`, include invalid lines as the third element of the tuple.
 	:param normalize_func: Function to use to normalize the names of requirements.
 
 	:return: The requirements, and a list of commented lines.
-
-	.. versionchanged:: 0.2.0 Added the ``include_invalid`` option.
-
-	.. versionchanged:: 0.2.1 Added the ``normalize_func`` keyword-only argument.
 	"""
 
 	comments = []
@@ -430,9 +428,9 @@ class RequirementsManager(ABC):
 		"""
 		Normalize the given name for PyPI et al.
 
-		:param name: The project name.
-
 		.. versionadded:: 0.2.1
+
+		:param name: The project name.
 		"""
 
 		return normalize(name)
@@ -471,7 +469,7 @@ class RequirementsManager(ABC):
 		"""
 
 		lib_requirements, _ = read_requirements(self.repo_path / "requirements.txt")
-		lib_requirements_names_extras = {normalize(r.name): r.extras for r in lib_requirements}
+		lib_requirements_names_extras = {normalize(r.name): r.extras for r in lib_requirements if not r.marker}
 
 		non_library_requirements = set()
 
@@ -479,6 +477,8 @@ class RequirementsManager(ABC):
 			if req.name in lib_requirements_names_extras:
 				if req.extras != lib_requirements_names_extras[req.name]:
 					non_library_requirements.add(req)
+				if req.marker:
+					non_library_requirements.add(req)
 			else:
 				non_library_requirements.add(req)
 
@@ -545,17 +545,17 @@ def list_requirements(name: str, depth: int = 1) -> Iterator[Union[str, List[str
 	try:
 		raw_deps = importlib_metadata.requires(req.name) or []
 	except importlib_metadata.PackageNotFoundError:
-		return ''
+		return
 
 	for requirement in [ComparableRequirement(r) for r in raw_deps]:
-		if not req.extras and requirement.marker:
-			continue
+		if requirement.marker:
+			if req.extras:
+				extras = list(req.extras)[0]
+			else:
+				extras = ''
 
-		if (
-				req.extras and requirement.marker
-				and not requirement.marker.evaluate(marker_environment(list(req.extras)[0]))
-				):
-			continue
+			if not requirement.marker.evaluate(marker_environment(extras)):
+				continue
 
 		if depth:
 			yield str(requirement)
